import store from '@/store'
import config from '@/config'
import { getToken, getTenantId } from '@/utils/auth'
import errorCode from '@/utils/errorCode'
import { toast, showConfirm, tansParams } from '@/utils/common'

let timeout = 10000
const baseUrl = config.baseUrl

const request = config => {
  // 是否需要设置 token
  const isToken = (config.headers || {}).isToken === false
  config.header = config.header || {}
  if (getToken() && !isToken) {
    config.header['Authorization'] = 'Bearer ' + getToken()
  }
  // get请求映射params参数
  if (config.params) {
    let url = config.url + '?' + tansParams(config.params)
    url = url.slice(0, -1)
    config.url = url
  }
	// 设置租户
	const isTanantId = (config.headers || {}).isTanantId === false
	config.header = config.header || {}
	if(getTenantId() && !isTanantId){
	  config.header['Tenant-Id'] = getTenantId()
	}
  return new Promise((resolve, reject) => {
    uni.request({
        method: config.method || 'get',
        timeout: config.timeout ||  timeout,
        url: config.baseUrl || baseUrl + config.url,
        data: config.data,
        header: config.header,
        dataType: 'json'
      }).then(response => {
		  // debugger
        let  res = response
  
        const code = res.data.code || 200
        const msg = errorCode[code] || res.data.msg || errorCode['default']
        if (code === 401) {
			  // uni.reLaunch({ url: '/pages/Login/login' })

      //         store.dispatch('LogOut').then(res => {
				  //   uni.reLaunch({ url: '/pages/Login/login' })
				  // })
							uni.clearStorageSync()
              uni.reLaunch({ url: '/pages/Login/login' })
				

        } else if (code === 500) {
          toast(msg)
          reject('500')
        } else if (code !== 200) {
          toast(msg)
          reject(code)
        }
				res.data.code = (res.data.code === 0 ? 200 : res.data.code)
				if(res.data.data && res.data.data.records){
				  res.data.rows = res.data.data.records
				}
				if(res.data.data && res.data.data.total){
				  res.data.total = res.data.data.total
				}
        resolve(res.data)
      })
      .catch(error => {
        let { message } = error
        if (message === 'Network Error') {
          message = '后端接口连接异常'
        } else if (message.includes('timeout')) {
          message = '系统接口请求超时'
        } else if (message.includes('Request failed with status code')) {
          message = '系统接口' + message.substr(message.length - 3) + '异常'
        }
        toast(message)
        reject(error)
      })
  })
}

export default request
